# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - MoinForms library

    @copyright: 2012 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin import wikiutil
from StringIO import StringIO
from MoinSupport import *
import re

__version__ = "0.1"

# Common formatting functions.

def formatForm(text, request, fmt, attrs=None, write=None):

    """
    Format the given 'text' using the specified 'request' and formatter 'fmt'.
    The optional 'attrs' can be used to control the presentation of the form.

    If the 'write' parameter is specified, use it to write output; otherwise,
    write output using the request.
    """

    write = write or request.write
    page = request.page

    fields = getFields(get_form(request))

    querystr = attrs and attrs.has_key("action") and ("action=%s" % attrs["action"]) or None

    write(fmt.rawHTML('<form method="post" action="%s">' %
        escattr(page.url(request, querystr))
        ))

    output = getFormOutput(text, fields)
    write(formatText(output, request, fmt))

    write(fmt.rawHTML('</form>'))

def getFormOutput(text, fields):

    """
    Combine regions found in the given 'text' and then return them as a single
    block. The reason for doing this, as opposed to just passing each region to
    a suitable parser for formatting, is that form sections may break up
    regions, and such sections may not define separate subregions but instead
    act as a means of conditional inclusion of text into an outer region.

    The given 'fields' are used to populate fields provided in forms and to
    control whether sections are populated or not.
    """

    output = []
    section = fields

    for region in getRegions(text, True):
        format, attributes, body, header, close = getFragmentFromRegion(region)

        # NOTE: Need to adjust FormField macros to use hierarchical names.

        # Include bare regions as they are.

        if format is None:
            output.append(region)

        # Include form sections only if fields exist for those sections.

        elif format == "form":
            section_name = attributes.get("section")
            if section_name and section.has_key(section_name):
                output.append(header)
                output.append(getFormOutput(body, section[section_name]))
                output.append(close)

        # Inspect and include other regions.

        else:
            output.append(header)
            output.append(getFormOutput(body, section))
            output.append(close)

    return "".join(output)

def getFields(d):

    """
    Return the form fields hierarchy for the given dictionary 'd'.
    """

    fields = {}

    for key, value in d.items():

        # Reproduce the original hierarchy of the fields.

        section = fields
        parts = key.split("/")

        for part in parts[:-1]:
            try:
                name, index = part.split("$", 1)
                index = int(index)
            except ValueError:
                name, index = part, None

            if not section.has_key(name):
                section[name] = {}

            if not section[name].has_key(index):
                section[name][index] = {}

            section = section[name][index]

        section[parts[-1]] = value

    return fields

def formatFormForOutputType(text, request, mimetype, attrs=None, write=None):

    """
    Format the given 'text' using the specified 'request' for the given output
    'mimetype'.

    The optional 'attrs' can be used to control the presentation of the form.

    If the 'write' parameter is specified, use it to write output; otherwise,
    write output using the request.
    """

    write = write or request.write

    if mimetype == "text/html":
        write('<html>')
        write('<body>')
        fmt = request.html_formatter
        fmt.setPage(request.page)
        formatForm(text, request, fmt, attrs, write)
        write('</body>')
        write('</html>')

# vim: tabstop=4 expandtab shiftwidth=4
